# iri_road_description

This package contains the urdf and launch files to load road images, which can be used in simulation.



## Installation

Move to workspace:
```
roscd && cd ../src
```

Clone the repository: 
```
git clone https://gitlab.iri.upc.edu/mobile_robotics/adc/adc_2021/iri_road_description.git
```

Install all ROS dependencies with the following commands:
```
roscd
cd ..
rosdep install -i -r --from-paths src
```

Compile all the ROS packages with the following commands:
```
catkin_make
```

## Configuration

No road image is provided in this repository due to its size.

An example road image can be download from [here](https://drive.google.com/file/d/14pwD1jBnt6BG-QkPzPemC-FtrFLofC4R/view?usp=sharing). 

To generate new road images, follow the next steps:

* Create a new road or edit an existing one using the [OpenRoadEd](https://gitlab.iri.upc.edu/mobile_robotics/adc/libraries/openroaded) tool. See the documentation on this link for more information.
* Create an image of the desired road using the take screenshot tool of the OpenRoadEd application. Together with the image, a YAML file is generated with the followng information:
  * **offset in x**: (in meters) position of the origin of the road with respect of the center of the image in the x direction.
  * **offset in y**: (in meters) position of the origin of the road with respect of the center of the image in the y direction.
  * **rotation**: (in radians) rotation of the road with respect of the center of the image (at the moment this value is always 0).
  * **real size:** (in meters) actual size of the road image.
* The screenshot is created as a 7000x7000 pixel image with a default resolution of 72 ppi (pixel per inch). Because of this fact, no matter what is the actual size of the real road, the screenshot equivalent size will be different. To *resize* the original image it is necessary to modify its ppi. To do so, compute the new ppi with the following equation, where *39.27* is the conversion from meters to inches, and the *real_size* can be taken from the YAML file:
 
<img align="center" src="docs/images/ppi_conv.gif" alt="Equation to compute the required ppi of the image" width="201" height="42">

* If you want to generate a scaled version of the orginal road, in the previous equation, use the road size from the YAML file scaled by the desired value. For example, if we want to create a 1:8 scale road, divide by 8 the actual road size.
* To change the ppi of the original image, execute the following command (both the original and modified images must be PNG):
```
convert <original_image> -density <new_ppi_value> -units PixelsPerInch <modified_image>
```

* The actual ppi of the new image may be not exactly the same as the desired one. To take this into account, the launch files accept a *road_size* parameter to set the actual size of the road. See the [launch](#launch) section.

* This packages includes a script to create a link to the actual road image, so it is not necessary to copy it anywhere. See the [launch](#launch) section.

## Launch

This package file includes a launch file named spawn_road.launch which is intended to be included from other launch file that requires to spawn a road in a simulated Gazebo world. This launch file has the following parameters:

* **road_name**: (default=road) name of the road image to be used (without the .png extension). This name will also be used to name the nodes and topics associated with the road description.
* **road_size**: (default=15.0) size of the road in meters. At the moment only square road images are supported. This value can be taken from the YAML file.
* **road_path**: (default=$(env HOME)/Downloads) path where the road image is located. It must be and absolute path.
* **x** (default=0.0) x position where the road will placed in the simulated environment with respect to the parent frame. This value can be taken from the YAML file. If you generated a scaled version of the original road, divide the YAML file value by the scale factor.
* **y** (default=0.0) y position where the road will placed in the simulated environment with respect to the parent frame. This value can be taken from the YAML file. If you generated a scaled version of the original road, divide the YAML file value by the scale factor.
* **yaw** (default=0.0) orientation of the road with respect to the parent frame. This value should normally be 1.5707 radians.
* **parent** (default=map) parent reference frame used to place the road in the simulated environment.

To spawn a road from an other launch file, include the following lines:
```
  <include file="$(find iri_road_description)/launch/spawn.launch">
    <arg name="road_name"  value="$(arg road_name)"/>
    <arg name="road_size"  value="$(arg road_size)"/>
    <arg name="road_path"  value="$(arg road_path)"/>
    <arg name="x"          value="$(arg x)"/>
    <arg name="y"          value="$(arg y)"/>
    <arg name="yaw"        value="$(arg yaw)"/>
    <arg name="parent"     value="$(arg parent)"/>
  </include>
```

To spawn the road in Gazebo, execute the following command:

```
roslaunch iri_road_description spawn_road.launch road_name:=<name of the road> road_path:=<path to the road> road_size:=<size of the road>`
```

To visualze the spawned road in RVIZ, execute the following command:

```
rosrun rviz rviz -d `rospack find iri_road_description`/rviz/road.rviz
```

## Disclaimer

Copyright (C) Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
Mantainer IRI labrobotics (labrobotica@iri.upc.edu)

This package is distributed in the hope that it will be useful, but without any warranty. It is provided "as is" without warranty of any kind, either expressed or implied, including, but not limited to, the implied warranties of merchantability and fitness for a particular purpose. The entire risk as to the quality and performance of the program is with you. should the program prove defective, the GMR group does not assume the cost of any necessary servicing, repair  or correction.

In no event unless required by applicable law the author will be liable to you for damages, including any general, special, incidental or consequential damages arising out of the use or inability to use the program (including but not limited to loss of data or data being rendered inaccurate or losses sustained by you or third parties or a failure of the program to operate with any other programs), even if the author has been advised of the possibility of such damages.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

